@@ -24,11 +24,12 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
+
+#include "gdkalias.h"
 #include "gdkx.h"
 #include "gdkregion-generic.h"
 
-#include <config.h>
-
 #include <pango/pangoxft.h>
 
 #include <stdlib.h>
@@ -817,7 +818,7 @@ gdk_x11_draw_image     (GdkDrawable     *drawable,
                         gint             height)
 {
   GdkDrawableImplX11 *impl;
-  
+
   impl = GDK_DRAWABLE_IMPL_X11 (drawable);
 
 #ifdef USE_SHM  
@@ -846,7 +847,7 @@ static GdkDrawable * get_impl_drawable (GdkDrawable *drawable)
   GdkDrawable *impl;
   
   if (GDK_IS_WINDOW (drawable))
-    impl = ((GdkPixmapObject *)drawable)->impl;
+    impl = ((GdkWindowObject *)drawable)->impl;
   else if (GDK_IS_PIXMAP (drawable))
     impl = ((GdkPixmapObject *)drawable)->impl;
   else
@@ -921,14 +922,8 @@ gdk_x11_drawable_get_xid (GdkDrawable *drawable)
  * what's the fastest depending on the available picture formats,
  * whether we can used shared pixmaps, etc.
  */
-typedef enum {
-  FORMAT_NONE,
-  FORMAT_EXACT_MASK,
-  FORMAT_ARGB_MASK,
-  FORMAT_ARGB
-} FormatType;
 
-static FormatType
+static GdkX11FormatType
 select_format (GdkDisplay         *display,
 	       XRenderPictFormat **format,
 	       XRenderPictFormat **mask)
@@ -937,7 +932,7 @@ select_format (GdkDisplay         *display,
   XRenderPictFormat pf;
 
   if (!_gdk_x11_have_render (display))
-    return FORMAT_NONE;
+    return GDK_X11_FORMAT_NONE;
   
   /* Look for a 32-bit xRGB and Axxx formats that exactly match the
    * in memory data format. We can use them as pixmap and mask
@@ -993,7 +988,7 @@ select_format (GdkDisplay         *display,
 			     0);
 
   if (*format && *mask)
-    return FORMAT_EXACT_MASK;
+    return GDK_X11_FORMAT_EXACT_MASK;
 
   /* OK, that failed, now look for xRGB and Axxx formats in
    * RENDER's preferred order
@@ -1023,7 +1018,7 @@ select_format (GdkDisplay         *display,
 			     0);
 
   if (*format && *mask)
-    return FORMAT_ARGB_MASK;
+    return GDK_X11_FORMAT_ARGB_MASK;
 
   /* Finally, if neither of the above worked, fall back to
    * looking for combined ARGB -- we'll premultiply ourselves.
@@ -1048,9 +1043,9 @@ select_format (GdkDisplay         *display,
   *mask = NULL;
 
   if (*format)
-    return FORMAT_ARGB;
+    return GDK_X11_FORMAT_ARGB;
 
-  return FORMAT_NONE;
+  return GDK_X11_FORMAT_NONE;
 }
 
 #if 0
@@ -1081,15 +1076,15 @@ list_formats (XRenderPictFormat *pf)
 }
 #endif  
 
-static void
-convert_to_format (guchar        *src_buf,
-		   gint           src_rowstride,
-		   guchar        *dest_buf,
-		   gint           dest_rowstride,
-		   FormatType     dest_format,
-		   GdkByteOrder   dest_byteorder,
-		   gint           width,
-		   gint           height)
+void
+_gdk_x11_convert_to_format (guchar           *src_buf,
+                            gint              src_rowstride,
+                            guchar           *dest_buf,
+                            gint              dest_rowstride,
+                            GdkX11FormatType  dest_format,
+                            GdkByteOrder      dest_byteorder,
+                            gint              width,
+                            gint              height)
 {
   gint i;
 
@@ -1097,14 +1092,14 @@ convert_to_format (guchar        *src_buf,
     {
       switch (dest_format)
 	{
-	case FORMAT_EXACT_MASK:
+	case GDK_X11_FORMAT_EXACT_MASK:
 	  {
 	    memcpy (dest_buf + i * dest_rowstride,
 		    src_buf + i * src_rowstride,
 		    width * 4);
 	    break;
 	  }
-	case FORMAT_ARGB_MASK:
+	case GDK_X11_FORMAT_ARGB_MASK:
 	  {
 	    guchar *row = src_buf + i * src_rowstride;
 	    if (((gsize)row & 3) != 0)
@@ -1182,7 +1177,7 @@ convert_to_format (guchar        *src_buf,
 	      }
 	    break;
 	  }
-	case FORMAT_ARGB:
+	case GDK_X11_FORMAT_ARGB:
 	  {
 	    guchar *p = (src_buf + i * src_rowstride);
 	    guchar *q = (dest_buf + i * dest_rowstride);
@@ -1218,7 +1213,7 @@ convert_to_format (guchar        *src_buf,
 #undef MULT
 	    break;
 	  }
-	case FORMAT_NONE:
+	case GDK_X11_FORMAT_NONE:
 	  g_assert_not_reached ();
 	  break;
 	}
@@ -1228,7 +1223,7 @@ convert_to_format (guchar        *src_buf,
 static void
 draw_with_images (GdkDrawable       *drawable,
 		  GdkGC             *gc,
-		  FormatType         format_type,
+		  GdkX11FormatType   format_type,
 		  XRenderPictFormat *format,
 		  XRenderPictFormat *mask_format,
 		  guchar            *src_rgb,
@@ -1273,10 +1268,10 @@ draw_with_images (GdkDrawable       *drawable,
 	  
 	  image = _gdk_image_get_scratch (screen, width1, height1, 32, &xs0, &ys0);
 	  
-	  convert_to_format (src_rgb + y0 * src_rowstride + 4 * x0, src_rowstride,
-			     (guchar *)image->mem + ys0 * image->bpl + xs0 * image->bpp, image->bpl,
-			     format_type, image->byte_order, 
-			     width1, height1);
+	  _gdk_x11_convert_to_format (src_rgb + y0 * src_rowstride + 4 * x0, src_rowstride,
+                                      (guchar *)image->mem + ys0 * image->bpl + xs0 * image->bpp, image->bpl,
+                                      format_type, image->byte_order, 
+                                      width1, height1);
 
 	  gdk_draw_image (pix, pix_gc,
 			  image, xs0, ys0, x0, y0, width1, height1);
@@ -1352,7 +1347,7 @@ get_shm_pixmap_for_image (Display           *xdisplay,
 static gboolean
 draw_with_pixmaps (GdkDrawable       *drawable,
 		   GdkGC             *gc,
-		   FormatType         format_type,
+		   GdkX11FormatType   format_type,
 		   XRenderPictFormat *format,
 		   XRenderPictFormat *mask_format,
 		   guchar            *src_rgb,
@@ -1386,10 +1381,10 @@ draw_with_pixmaps (GdkDrawable       *drawable,
 	  if (!get_shm_pixmap_for_image (xdisplay, image, format, mask_format, &pix, &pict, &mask))
 	    return FALSE;
 
-	  convert_to_format (src_rgb + y0 * src_rowstride + 4 * x0, src_rowstride,
-			     (guchar *)image->mem + ys0 * image->bpl + xs0 * image->bpp, image->bpl,
-			     format_type, image->byte_order, 
-			     width1, height1);
+	  _gdk_x11_convert_to_format (src_rgb + y0 * src_rowstride + 4 * x0, src_rowstride,
+                                      (guchar *)image->mem + ys0 * image->bpl + xs0 * image->bpp, image->bpl,
+                                      format_type, image->byte_order, 
+                                      width1, height1);
 
 	  XRenderComposite (xdisplay, PictOpOver, pict, mask, dest_pict, 
 			    xs0, ys0, xs0, ys0, x0 + dest_x, y0 + dest_y,
@@ -1415,7 +1410,7 @@ gdk_x11_draw_pixbuf (GdkDrawable     *drawable,
 		     gint             x_dither,
 		     gint             y_dither)
 {
-  FormatType format_type;
+  GdkX11FormatType format_type;
   XRenderPictFormat *format, *mask_format;
   gint rowstride;
 #ifdef USE_SHM  
@@ -1425,7 +1420,7 @@ gdk_x11_draw_pixbuf (GdkDrawable     *drawable,
   format_type = select_format (gdk_drawable_get_display (drawable),
 			       &format, &mask_format);
 
-  if (format_type == FORMAT_NONE ||
+  if (format_type == GDK_X11_FORMAT_NONE ||
       !gdk_pixbuf_get_has_alpha (pixbuf) ||
       gdk_drawable_get_depth (drawable) == 1 ||
       (dither == GDK_RGB_DITHER_MAX && gdk_drawable_get_depth (drawable) != 24) ||
@@ -1462,3 +1457,68 @@ gdk_x11_draw_pixbuf (GdkDrawable     *drawable,
 		      rowstride,
 		      dest_x, dest_y, width, height);
 }
+
+/**
+ * gdk_draw_rectangle_alpha_libgtk_only:
+ * @drawable: The #GdkDrawable to draw on
+ * @x: the x coordinate of the left edge of the rectangle.
+ * @y: the y coordinate of the top edge of the rectangle.
+ * @width: the width of the rectangle.
+ * @height: the height of the rectangle.
+ * @color: The color
+ * @alpha: The alpha value.
+ * 
+ * Tries to draw a filled alpha blended rectangle using the window
+ * system's native routines. This is not public API and must not be
+ * used by applications.
+ * 
+ * Return value: TRUE if the rectangle could be drawn, FALSE
+ * otherwise.
+ **/
+gboolean
+gdk_draw_rectangle_alpha_libgtk_only (GdkDrawable  *drawable,
+				      gint          x,
+				      gint          y,
+				      gint          width,
+				      gint          height,
+				      GdkColor     *color,
+				      guint16       alpha)
+{
+  Display *xdisplay;
+  XRenderColor render_color;
+  Picture pict;
+  int x_offset, y_offset;
+  GdkDrawable *real_drawable, *impl;
+
+  g_return_val_if_fail (color != NULL, FALSE);
+  
+  if (!GDK_IS_WINDOW (drawable))
+    return FALSE;
+  
+  if (!_gdk_x11_have_render (gdk_drawable_get_display (drawable)))
+    return FALSE;
+
+  gdk_window_get_internal_paint_info (GDK_WINDOW (drawable),
+				      &real_drawable,
+				      &x_offset, &y_offset);
+
+  impl = ((GdkWindowObject *)real_drawable)->impl;  
+      
+  pict = gdk_x11_drawable_get_picture (impl);
+
+  if (pict == None)
+    return FALSE;
+  
+  xdisplay = GDK_DISPLAY_XDISPLAY (gdk_drawable_get_display (drawable));
+
+  render_color.alpha = alpha;
+  render_color.red = (guint32)color->red * render_color.alpha / 0xffff;
+  render_color.green = (guint32)color->green * render_color.alpha / 0xffff;
+  render_color.blue = (guint32)color->blue * render_color.alpha / 0xffff;
+
+  XRenderFillRectangle (xdisplay,
+			PictOpOver, pict, &render_color,
+			x - x_offset, y - y_offset,
+			width, height);
+  return TRUE;
+}
