@@ -27,6 +27,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <glib.h>
+#include "gtkalias.h"
 
 #ifdef G_OS_WIN32
 #ifndef S_ISDIR
@@ -40,6 +41,8 @@
 #include "gtksettings.h"
 #include "gtkprivate.h"
 
+
+
 #define DEFAULT_THEME_NAME "hicolor"
 
 typedef struct _GtkIconData GtkIconData;
@@ -521,24 +524,31 @@ static void
 gtk_icon_theme_init (GtkIconTheme *icon_theme)
 {
   GtkIconThemePrivate *priv;
-
+  const gchar * const *xdg_data_dirs;
+  int i, j;
+  
   priv = g_type_instance_get_private ((GTypeInstance *)icon_theme,
 				      GTK_TYPE_ICON_THEME);
   icon_theme->priv = priv;
 
   priv->custom_theme = FALSE;
   priv->current_theme = g_strdup (DEFAULT_THEME_NAME);
-  priv->search_path = g_new (char *, 5);
-  
 
-  priv->search_path[0] = g_build_filename (g_get_home_dir (),
-					   ".icons",
-					   NULL);
-  priv->search_path[1] = g_build_filename (GTK_DATADIR, "pixmaps", NULL);
-  priv->search_path[2] = g_build_filename (GTK_DATADIR, "icons", NULL);
-  priv->search_path[3] = g_strdup ("/usr/share/icons");
-  priv->search_path[4] = g_strdup ("/usr/share/pixmaps");
-  priv->search_path_len = 5;
+  xdg_data_dirs = g_get_system_data_dirs ();
+  for (i = 0; xdg_data_dirs[i]; i++) ;
+
+  priv->search_path_len = i + 3;
+  
+  priv->search_path = g_new (char *, priv->search_path_len);
+  
+  i = 0;
+  priv->search_path[i++] = g_build_filename (g_get_home_dir (), ".icons", NULL);
+  priv->search_path[i++] = g_build_filename (g_get_user_data_dir (), "icons", NULL);
+  
+  for (j = 0; xdg_data_dirs[j]; j++) 
+    priv->search_path[i++] = g_build_filename (xdg_data_dirs[j], "icons", NULL);
+
+  priv->search_path[i++] = g_strdup ("/usr/share/pixmaps");
 
   priv->themes_valid = FALSE;
   priv->themes = NULL;
@@ -560,7 +570,7 @@ do_theme_change (GtkIconTheme *icon_theme)
   GtkIconThemePrivate *priv = icon_theme->priv;
   
   blow_themes (icon_theme);
-  g_signal_emit (G_OBJECT (icon_theme), signal_changed, 0);
+  g_signal_emit (icon_theme, signal_changed, 0);
   
   if (priv->screen && priv->is_screen_singleton)
     {
@@ -1304,7 +1314,7 @@ add_size (gpointer  key,
 
   **res_p = GPOINTER_TO_INT (key);
 
-  *res_p++;
+  (*res_p)++;
 }
 
 /**
@@ -1328,7 +1338,7 @@ gtk_icon_theme_get_icon_sizes (GtkIconTheme *icon_theme,
 {
   GList *l, *d;
   GHashTable *sizes;
-  gint *result;
+  gint *result, *r;
   guint suffix;
   
   GtkIconThemePrivate *priv;
@@ -1359,9 +1369,9 @@ gtk_icon_theme_get_icon_sizes (GtkIconTheme *icon_theme,
 	}
     }
 
-  result = g_new0 (gint, g_hash_table_size (sizes) + 1);
+  r = result = g_new0 (gint, g_hash_table_size (sizes) + 1);
 
-  g_hash_table_foreach (sizes, add_size, &result);
+  g_hash_table_foreach (sizes, add_size, &r);
   g_hash_table_destroy (sizes);
   
   return result;
@@ -2195,7 +2205,7 @@ load_svg_at_size (const gchar *filename,
 {
   GdkPixbuf *pixbuf = NULL;
   GdkPixbufLoader *loader = NULL;
-  gchar *contents;
+  gchar *contents = NULL;
   gsize length;
   
   if (!g_file_get_contents (filename,
@@ -2219,6 +2229,8 @@ load_svg_at_size (const gchar *filename,
  bail:
   if (loader)
     g_object_unref (loader);
+  if (contents)
+    g_free (contents);
   
   return pixbuf;
 }
