@@ -31,6 +31,7 @@
 #include <config.h>
 
 #include <string.h>
+#include "gtkalias.h"
 #include "gtkintl.h"
 #include "gtkmarshalers.h"
 #include "gtkmenu.h"
@@ -689,7 +690,7 @@ gtk_ui_manager_get_accel_group (GtkUIManager *self)
  * separated by '/'. Elements which don't have a name or action attribute in 
  * the XML (e.g. &lt;popup&gt;) can be addressed by their XML element name 
  * (e.g. "popup"). The root element ("/ui") can be omitted in the path.
- * 
+ *
  * Note that the widget found by following a path that ends in a &lt;menu&gt;
  * element is the menuitem to which the menu is attached, not the menu itself.
  * 
@@ -915,7 +916,7 @@ get_node (GtkUIManager *self,
   
   if (strncmp ("/ui", path, 3) == 0)
     path += 3;
-
+  
   end = path + strlen (path);
   pos = path;
   parent = node = NULL;
@@ -934,7 +935,7 @@ get_node (GtkUIManager *self,
 			     create, FALSE);
       if (!node)
 	return NULL;
-      
+
       pos += length + 1; /* move past the node name and the slash too */
       parent = node;
     }
@@ -1631,7 +1632,7 @@ gtk_ui_manager_add_ui (GtkUIManager        *self,
 	  node_type = NODE_TYPE_SEPARATOR;
 	  break;
 	case GTK_UI_MANAGER_PLACEHOLDER:
-	  node_type = NODE_TYPE_MENU_PLACEHOLDER;
+	  node_type = NODE_TYPE_TOOLBAR_PLACEHOLDER;
 	  break;
 	default: ;
 	  /* do nothing */
@@ -2672,12 +2673,15 @@ print_node (GtkUIManager *self,
   g_string_append_printf (buffer, open_tag_format[mnode->type],
 			  indent_level, "");
 
-  if (mnode->name)
-    g_string_append_printf (buffer, " name=\"%s\"", mnode->name);
-
-  if (mnode->action_name)
-    g_string_append_printf (buffer, " action=\"%s\"",
-                            g_quark_to_string (mnode->action_name));
+  if (mnode->type != NODE_TYPE_ROOT)
+    {
+      if (mnode->name)
+	g_string_append_printf (buffer, " name=\"%s\"", mnode->name);
+      
+      if (mnode->action_name)
+	g_string_append_printf (buffer, " action=\"%s\"",
+				g_quark_to_string (mnode->action_name));
+    }
 
   g_string_append (buffer,
                    close_tag_format[mnode->type] ? ">\n" : "/>\n");
