@@ -1,4 +1,4 @@
-#include "config.h"
+#include <config.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -66,9 +66,8 @@ toggle_tearoffs (GtkWidget    *button,
   gtk_ui_manager_set_add_tearoffs (merge, !add_tearoffs);
 }
 
-static void
-toggle_dynamic (GtkWidget    *button, 
-		GtkUIManager *merge)
+static gint
+delayed_toggle_dynamic (GtkUIManager *merge)
 {
   GtkAction *dyn;
   static GtkActionGroup *dynamic = NULL;
@@ -114,8 +113,16 @@ toggle_dynamic (GtkWidget    *button,
       gtk_ui_manager_remove_ui (merge, merge_id);
       merge_id = 0;
     }
+
+  return FALSE;
 }
 
+static void
+toggle_dynamic (GtkWidget    *button, 
+		GtkUIManager *merge)
+{
+  g_timeout_add (2000, delayed_toggle_dynamic, merge);
+}
 
 static void
 activate_action (GtkAction *action)
@@ -682,8 +689,41 @@ main (int argc, char **argv)
   gtk_widget_show_all (window);
   gtk_main ();
 
-  g_object_unref (action_group);
+#ifdef DEBUG_UI_MANAGER
+  {
+    GList *action;
+    
+    g_print ("\n> before unreffing the ui manager <\n");
+    for (action = gtk_action_group_list_actions (action_group);
+	 action; 
+	 action = action->next)
+      {
+	GtkAction *a = action->data;
+	g_print ("  action %s ref count %d\n", 
+		 gtk_action_get_name (a), G_OBJECT (a)->ref_count);
+      }
+  }
+#endif
+
   g_object_unref (merge);
 
+#ifdef DEBUG_UI_MANAGER
+  {
+    GList *action;
+
+    g_print ("\n> after unreffing the ui manager <\n");
+    for (action = gtk_action_group_list_actions (action_group);
+	 action; 
+	 action = action->next)
+      {
+	GtkAction *a = action->data;
+	g_print ("  action %s ref count %d\n", 
+		 gtk_action_get_name (a), G_OBJECT (a)->ref_count);
+      }
+  }
+#endif
+
+  g_object_unref (action_group);
+
   return 0;
 }
